/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


#include <windows.h>
#include <windowsx.h>

#include "plugin.h"
#include "lesson7.h"

//////////////////////////////////////
//
// general initialization and shutdown
//
NPError NS_PluginInitialize()
{
  return NPERR_NO_ERROR;
}

void NS_PluginShutdown()
{
}

/////////////////////////////////////////////////////////////
//
// construction and destruction of our plugin instance object
//
nsPluginInstanceBase * NS_NewPluginInstance(nsPluginCreateData * aCreateDataStruct)
{
  if(!aCreateDataStruct)
    return NULL;

  nsPluginInstance * plugin = new nsPluginInstance(aCreateDataStruct->instance);
  return plugin;
}

void NS_DestroyPluginInstance(nsPluginInstanceBase * aPlugin)
{
  if(aPlugin)
    delete (nsPluginInstance *)aPlugin;
}

////////////////////////////////////////
//
// nsPluginInstance class implementation
//
nsPluginInstance::nsPluginInstance(NPP aInstance) : nsPluginInstanceBase(),
  mInstance(aInstance),
  mInitialized(FALSE)
{
  mhWnd = NULL;
}

nsPluginInstance::~nsPluginInstance()
{
}

extern HDC		hGL_DC;		// Private GDI Device Context
extern HGLRC	hGL_RC;		// Permanent Rendering Context
extern HWND		hGL_Wnd;	// Holds Our Window Handle

static LRESULT CALLBACK PluginWinProc(HWND, UINT, WPARAM, LPARAM);
static WNDPROC lpOldProc = NULL;

BOOL set_opengl_contex(int width, int height){
	static	PIXELFORMATDESCRIPTOR pfd=				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		32,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if (!(hGL_DC=GetDC(hGL_Wnd))){
		KillGLWindow();
		MessageBox(NULL,"Can't Get A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
		}

	int PixelFormat = 0;
	if (!(PixelFormat=ChoosePixelFormat(hGL_DC,&pfd))){
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
		}

	if(!SetPixelFormat(hGL_DC,PixelFormat,&pfd)){
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
		}

	if (!(hGL_RC=wglCreateContext(hGL_DC)))	{
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
		}

	if(!wglMakeCurrent(hGL_DC,hGL_RC)){
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
		}

	ReSizeGLScene(width, height);

	if (!InitGL()){
		KillGLWindow();								// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
		}

	return TRUE;
}

struct TGLThread{
	HANDLE hThread;
	};

const WM_GL_MESSAGE = WM_APP + 0x435;
DWORD WINAPI gl_draw_thread(void* pv){
	if(!pv) return 0;
	TGLThread *sd = (TGLThread*)pv;
	HANDLE hTemp = sd->hThread;
	delete sd;

	SetForegroundWindow(hGL_Wnd);					// Slightly Higher Priority
	SetFocus(hGL_Wnd);

	int n = 0;
	//*
	while(hGL_DC && hGL_RC && hGL_Wnd){
		::PostMessage(hGL_Wnd,WM_GL_MESSAGE,0,0);
		Sleep(50);
		}
	//*/
	CloseHandle(hTemp);
	//ShowMsg("Done");  
	 
	return 0;
}

NPBool nsPluginInstance::init(NPWindow* aWindow)
{
  if(aWindow == NULL)
    return FALSE;

  mhWnd = (HWND)aWindow->window;
  if(mhWnd == NULL)
    return FALSE;

  // subclass window so we can intercept window messages and
  // do our drawing to it

  // associate window with our nsPluginInstance object so we can access 
  // it in the window procedure


  if(!hGL_Wnd){
   lpOldProc = SubclassWindow(mhWnd, (WNDPROC)PluginWinProc);
	hGL_Wnd = mhWnd;
	SetWindowLong(mhWnd, GWL_USERDATA, (LONG)this);
	set_opengl_contex(aWindow->width,aWindow->height);
	TGLThread* pData = new TGLThread();
	DWORD	dwThreadID;
	pData->hThread = ::CreateThread(NULL,0, &gl_draw_thread, (void*)pData, 0, &dwThreadID);
	}

  mInitialized = TRUE;
  return TRUE;
}

void nsPluginInstance::shut()
{
  // subclass it back
  KillGLWindow();
  SubclassWindow(mhWnd, lpOldProc);
  mhWnd = NULL;
  mInitialized = FALSE;
}

NPBool nsPluginInstance::isInitialized()
{
  return mInitialized;
}

const char * nsPluginInstance::getVersion()
{
  return NPN_UserAgent(mInstance);
}

static LRESULT CALLBACK PluginWinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg) {
	case WM_GL_MESSAGE:
		//if(wglMakeCurrent(hGL_DC,hGL_RC)){
			DrawGLScene();
			SwapBuffers(hGL_DC);
		//	}
		break;
	case WM_ERASEBKGND:
		return 1;
		break;
    case WM_PAINT:
      {
		if(hGL_DC){
			DrawGLScene();
			SwapBuffers(hGL_DC);
			}
		else{
			// draw a frame and display the string
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hWnd, &ps);
			RECT rc;
			GetClientRect(hWnd, &rc);
			FrameRect(hdc, &rc, GetStockBrush(BLACK_BRUSH));

			// get our plugin instance object and ask it for the version string
			nsPluginInstance *plugin = (nsPluginInstance *)GetWindowLong(hWnd, GWL_USERDATA);
			if (plugin) {
			  const char * string = plugin->getVersion();
			  DrawText(hdc, string, strlen(string), &rc, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
			}
			else {
			  char string[] = "Error occured";
			  DrawText(hdc, string, strlen(string), &rc, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
			}

			EndPaint(hWnd, &ps);
			}
	  }
      break;
    default:
      break;
  }

  return WndProc(hWnd, msg, wParam, lParam);
}
